<?php
/**
 * MyPDO
 * @author Jason.Wei <jasonwei06@hotmail.com>
 * @license http://www.sunbloger.com/
 * @version 4.0 gbk
 */
class MyPDO
{
    protected static $dbName = '';
    private static $dsn;
    private static $dbh;
    
    /**
     * Connect
     *
     * @param String $dbHost 主机地址
     * @param String $dbUser 用户名
     * @param String $dbPassword 密码
     * @param String $dbName 数据库名
     * @param String $dbCharset 字符集
     */
    public static function connect($dbHost, $dbUser, $dbPassword, $dbName, $dbCharset = 'gbk')
    {
        try {
            self::$dsn = 'mysql:host='.$dbHost.';dbname='.$dbName;
            self::$dbh = new PDO(self::$dsn, $dbUser, $dbPassword);
            self::$dbh->exec("SET character_set_connection=$dbCharset, character_set_results=$dbCharset, character_set_client=binary");
        } catch (PDOException $e) {
            self::outputError($e->getMessage());
        }
    }
    
    /**
     * Query 查询
     *
     * @param String $strSql SQL语句
     * @param String $queryMode 查询方式(All or Row)
     * @param Boolean $debug
     * @return Array
     */
    public static function query($strSql, $queryMode = 'All', $debug = false)
    {
        if ($debug === true) self::debug($strSql);
        $recordset = self::$dbh->query($strSql);
        self::getPDOError();
        if ($recordset) {
            $recordset->setFetchMode(PDO::FETCH_ASSOC);
            if ($queryMode == 'All') {
                $result = $recordset->fetchAll();
            } elseif ($queryMode == 'Row') {
                $result = $recordset->fetch();
            }
        } else {
            return null;
        }
        return $result;
    }
    
    /**
     * Update 更新
     *
     * @param String $table 表名
     * @param Array $arrayDataValue 字段与值
     * @param String $where 条件
     * @param Boolean $debug
     * @return Int
     */
    public static function update($table, $arrayDataValue, $where = '', $debug = false)
    {
        self::checkFields($table, $arrayDataValue);
        if ($where) {
            $strSql = '';
            foreach ($arrayDataValue as $key => $value) {
                $strSql .= ", `$key`='$value'";
            }
            $strSql = substr($strSql, 1);
            $strSql = "UPDATE `$table` SET $strSql WHERE $where";
        } else {
            $strSql = "REPLACE INTO `$table` (`".implode('`,`', array_keys($arrayDataValue))."`) VALUES ('".implode("','", $arrayDataValue)."')";
        }
        if ($debug === true) self::debug($strSql);
        $result = self::$dbh->exec($strSql);
        self::getPDOError();
        return $result;
    }
    
    /**
     * Insert 插入
     *
     * @param String $table 表名
     * @param Array $arrayDataValue 字段与值
     * @param Boolean $debug
     * @return Int
     */
    public static function insert($table, $arrayDataValue, $debug = false)
    {
        self::checkFields($table, $arrayDataValue);
        $strSql = "INSERT INTO `$table` (`".implode('`,`', array_keys($arrayDataValue))."`) VALUES ('".implode("','", $arrayDataValue)."')";
        if ($debug === true) self::debug($strSql);
        $result = self::$dbh->exec($strSql);
        self::getPDOError();
        return $result;
    }
    
    /**
     * Replace 覆盖方式插入
     *
     * @param String $table 表名
     * @param Array $arrayDataValue 字段与值
     * @param Boolean $debug
     * @return Int
     */
    public static function replace($table, $arrayDataValue, $debug = false)
    {
        self::checkFields($table, $arrayDataValue);
        $strSql = "REPLACE INTO `$table`(`".implode('`,`', array_keys($arrayDataValue))."`) VALUES ('".implode("','", $arrayDataValue)."')";
        if ($debug === true) self::debug($strSql);
        $result = self::$dbh->exec($strSql);
        self::getPDOError();
        return $result;
    }
    
    /**
     * Delete 删除
     *
     * @param String $table 表名
     * @param String $where 条件
     * @param Boolean $debug
     * @return Int
     */
    public static function delete($table, $where = '', $debug = false)
    {
        if ($where == '') {
            exit('缺少必要条件');
        } else {
            $strSql = "DELETE FROM `$table` WHERE $where";
            if ($debug === true) self::debug($strSql);
            $result = self::$dbh->exec($strSql);
            self::getPDOError();
            return $result;
        }
    }
    
    /**
     * execSql 执行SQL语句
     *
     * @param String $strSql
     * @param Boolean $debug
     * @return Int
     */
    public static function execSql($strSql, $debug = false)
    {
        if ($debug === true) self::debug($strSql);
        $result = self::$dbh->exec($strSql);
        self::getPDOError();
        return $result;
    }
    
    /**
     * 获取字段最大值
     * 
     * @param string $table
     * @param string $field_name
     * @param string $where
     */
    public function getMaxValue($table, $field_name, $where = '', $debug = false)
    {
        $strSql = "SELECT MAX(".$field_name.") AS MAX_VALUE FROM $table";
        if ($where != "") $strSql .= " WHERE $where";
        if ($debug === true) self::debug($strSql);
        $arrTemp = db::query($strSql, 'Row');
        $max_value = $arrTemp["MAX_VALUE"];
        if ($max_value == "" || $max_value == null) {
            $max_value = 0;
        }
        return $max_value;
    }
    
    /**
     * 获取指定列的数量
     * 
     * @param string $table
     * @param string $field_name
     * @param string $where
     * @param bool $debug
     * @return int
     */
    public static function getCount($table, $field_name, $where = '', $debug = false)
    {
        $strSql = "SELECT COUNT($field_name) AS NUM FROM $table";
        if ($where != '') $strSql .= " WHERE $where";
        if ($debug === true) self::debug($strSql);
        $arrTemp = db::query($strSql, 'Row');
        return $arrTemp['NUM'];
    }
    
    /**
     * 获取表引擎
     * 
     * @param String $dbName 库名
     * @param String $tableName 表名
     * @param Boolean $debug
     * @return String
     */
    public static function getTableEngine($dbName, $tableName)
    {
        $strSql = "SHOW TABLE STATUS FROM $dbName WHERE Name='".$tableName."'";
        $arrayTableInfo = self::query($strSql);
        self::getPDOError();
        return $arrayTableInfo[0]['Engine'];
    }
    
    /**
     * beginTransaction 事务开始
     */
    private static function beginTransaction()
    {
        self::$dbh->beginTransaction();
    }
    
    /**
     * commit 事务提交
     */
    private static function commit()
    {
        self::$dbh->commit();
    }
    
    /**
     * rollback 事务回滚
     */
    private static function rollback()
    {
        self::$dbh->rollback();
    }
    
    /**
     * transaction 通过事务处理多条SQL语句
     * 调用前需通过getTableEngine判断表引擎是否支持事务
     *
     * @param array $arraySql
     * @return Boolean
     */
    public static function execTransaction($arraySql)
    {
        $retval = 1;
        self::beginTransaction();
        foreach ($arraySql as $strSql) {
            if (self::execSql($strSql) == 0) $retval = 0;
        }
        if ($retval == 0) {
            self::rollback();
            return false;
        } else {
            self::commit();
            return true;
        }
    }

    /**
     * checkFields 检查指定字段是否在指定数据表中存在
     *
     * @param String $table
     * @param array $arrayField
     */
    private static function checkFields($table, $arrayFields)
    {
        $fields = self::getFields($table);
        foreach ($arrayFields as $key => $value) {
            if (!in_array($key, $fields)) {
                self::outputError("Unknown column `$key` in field list.");
            }
        }
    }
    
    /**
     * getFields 获取指定数据表中的全部字段名
     *
     * @param String $table 表名
     * @return array
     */
    private static function getFields($table)
    {
        $fields = array();
        $recordset = self::$dbh->query("SHOW COLUMNS FROM $table");
        self::getPDOError();
        $recordset->setFetchMode(PDO::FETCH_ASSOC);
        $result = $recordset->fetchAll();
        foreach ($result as $rows) {
            $fields[] = $rows['Field'];
        }
        return $fields;
    }
    
    /**
     * getPDOError 捕获PDO错误信息
     */
    private static function getPDOError()
    {
        if (self::$dbh->errorCode() != '00000') {
            $arrayError = self::$dbh->errorInfo();
            self::outputError($arrayError[2]);
        }
    }
    
    /**
     * debug
     * 
     * @param mixed $debuginfo
     */
    private static function debug($debuginfo)
    {
        print_r($debuginfo);
        exit();
    }
    
    /**
     * 输出错误信息
     * 
     * @param String $strErrMsg
     */
    private static function outputError($strErrMsg)
    {
        exit('MySQL Error: '.$strErrMsg);
    }
    
    /**
     * destruct 关闭数据库连接
     */
    public static function destruct()
    {
        self::$dbh = null;
    }
}
?>